-- ChatTimestamps by Varstahl
--

local CTS_MainFrame;
local CTS_Hook_FCF_OTW;
local CTS_Frames = {};

function ChatTimestamps_Print(msg)
    if (DEFAULT_CHAT_FRAME) then
      DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 1.0, 0);
    else
      UIErrorsFrame:AddMessage(msg, 1.0, 1.0, 0, 1.0, UIERRORS_HOLD_TIME);
    end
end

function ChatTimestamps_AddTimeStamp(msg)
	if not (msg == nil) then
          local locFormat = ChatTimestampsSettings.Format;
          if (strfind(locFormat,"#H") ~= 1) then
            local ParsedTime = date("%H");
            if (strsub(ParsedTime,1,1) == "0") then ParsedTime = strsub(ParsedTime,2); end;
            locFormat = gsub(locFormat, "#H", ParsedTime);
          end
          if (strfind(locFormat,"#I") ~= 1) then
            local ParsedTime = date("%I");
            if (strsub(ParsedTime,1,1) == "0") then ParsedTime = strsub(ParsedTime,2); end;
            locFormat = gsub(locFormat, "#I", ParsedTime);
          end
	  msg = date(locFormat).." "..msg;  -- TimeStamp Format
	end
	return msg;
end

function ChatTimestamps_ChatFrame_AddMessage(frame, msg, ...)
  if CTS_Frames and not CTS_Frames[frame] then return end
  CTS_Frames[frame](frame, ChatTimestamps_AddTimeStamp(msg), ...);
--  CTS_Frames[frame].AddMessage(frame, , ...);
end

function ChatTimestamps_OnEvent()
             if not (ChatTimestampsSettings) then
               -- Default Settings
               ChatTimestampsSettings               = { };
               ChatTimestampsSettings.Format        = "[%H:%M:%S]";
             end

             ChatTimestamps_HookAllChatFrames();
end

function ChatTimestamps_OnLoad()

        -- Register the slash command
        SlashCmdList["CHATTIMESTAMPS"] = function(msg)
          ChatTimestamps_SlashCommand(msg);
        end;
        SLASH_CHATTIMESTAMPS1 = "/chattimestamps";
        SLASH_CHATTIMESTAMPS2 = "/cts";

        CTS_MainFrame = CreateFrame("Frame", "ChatTimestampsFrame");
        CTS_MainFrame:SetScript("OnEvent", ChatTimestamps_OnEvent);
        CTS_MainFrame:RegisterEvent("VARIABLES_LOADED");

        ChatTimestamps_Print("ChatTimestamps 1.7 loaded");
end

function ChatTimestamps_ProcessFrame(frame)
  if not CTS_Frames[frame] then
    CTS_Frames[frame] = frame.AddMessage
    frame.AddMessage = ChatTimestamps_ChatFrame_AddMessage;
  end
  return frame
end

function ChatTimestamps_HookAllChatFrames()
  -- Add static windows
  for i=1,NUM_CHAT_WINDOWS do
    local frame = getglobal("ChatFrame"..i);
    CTS_Frames[frame] = frame.AddMessage
    frame.AddMessage = ChatTimestamps_ChatFrame_AddMessage;
  end

  -- Add dynamically created windows
  CTS_Hook_FCF_OTW = FCF_OpenTemporaryWindow
  FCF_OpenTemporaryWindow = function(...)
      local frame = CTS_Hook_FCF_OTW(...);
      return ChatTimestamps_ProcessFrame(frame)
  end
end

function ChatTimestamps_SlashCommand(msg)

        -- Check the command
        if (msg) then
                local command = strtrim(msg);
                local params = "";

                if (strfind(command," ") ~= nul) then
                  command = strsub(command,0,strfind(command," ")-1);
                  params  = strtrim(strsub(msg,strfind(msg,command)+strlen(command)));
                  command = strlower(command);
                end

                if (command == "status") then
                        ChatTimestamps_Print("ChatTimestamps Format: "..ChatTimestampsSettings.Format);

                elseif ((command == "format") and (strlen(params) > 0)) then
                        params = strreplace(params,"||","|");
                        ChatTimestampsSettings.Format = params;
                        ChatTimestamps_Print("ChatTimestamps Format set to: \""..ChatTimestampsSettings.Format.."\"");

                else
                        ChatTimestamps_Print("ChatTimestamps Options:");
                        ChatTimestamps_Print("/cts status - Shows current settings");
                        ChatTimestamps_Print("/cts format <timeformat> (default \"[%H:%M:%S]\") - Sets timestamp format");
                end
        end
end

ChatTimestamps_OnLoad();